package it.unibs.fp.patenteelibbrettopregoh;

import java.time.LocalDate;
import java.util.ArrayList;

/**
 * GameData.java
 * questa classe rappresenta i dati di gioco, come il formato, il nome del personaggio,
 * il diario, le persone, il saldo, il numero di mazzette accettate dalla polizia,
 * il numero di rivoluzionari accettati, lo stato del gioco e la data.
 *
 */
public class GameData {
    private final Formato formato;
    private final String nomePersonaggio;
    private final ArrayList<PaginaDiario> diario;
    private final Comuni comuni;
    private final ArrayList<Persona> persone;
    private double saldo;
    private int numeroMazzetteAccettatePolizia;
    private int numeroRivoluzionariAccettati;
    private StatoGioco statoGioco;
    private int giorno;
    private LocalDate data;

    /**
     * costruttore della classe GameData.
     * Inizializza i dati di gioco con il formato specificato.
     *
     * @param formato il formato del gioco
     */
    public GameData(Formato formato) {
        this.formato = formato;
        this.diario = new ArrayList<>();
        this.comuni = Parser.parseComuni(formato, "FileEsercizio/comuni");
        this.persone = Parser.parsePersone(formato, "FileEsercizio/IDpersone");
        this.saldo = 0;
        this.numeroMazzetteAccettatePolizia = 0;
        this.numeroRivoluzionariAccettati = 0;
        this.statoGioco = StatoGioco.IN_CORSO;
        this.nomePersonaggio = "Ajeje Brazorf";
        this.giorno = 1;
        this.data = LocalDate.of(2024, 4, 26);
    }

    /**
     * getter per il diario.
     *
     */
    public ArrayList<PaginaDiario> getDiario() {
        return diario;
    }

    /**
     * getter per i comuni.
     *
     * @return i comuni
     */
    public Comuni getComuni() {
        return comuni;
    }

    /**
     * getter per le persone.
     *
     * @return le persone
     */
    public ArrayList<Persona> getPersone() {
        return persone;
    }

    /**
     * getter per il saldo.
     *
     * @return il saldo
     */
    public double getSaldo() {
        return saldo;
    }

    /**
     * getter di una persona casuale da controllare.
     *
     * @return la persona casuale da controllare
     */
    public Persona getPersonaDaControlare() {
        return persone.get((int) (Math.random() * (persone.size() + 1)));
    }

    /**
     * getter per il numero di mazzette accettate dalla polizia.
     *
     * @return il numero di mazzette accettate dalla polizia
     */
    public int getNumeroMazzetteAccettatePolizia() {
        return numeroMazzetteAccettatePolizia;
    }

    /**
     * getter per il numero di rivoluzionari accettati.
     *
     * @return il numero di rivoluzionari accettati
     */
    public int getNumeroRivoluzionariAccettati() {
        return numeroRivoluzionariAccettati;
    }

    /**
     * aggiunge una pagina al diario.
     */
    public void addPaginaDiario(PaginaDiario pagina) {
        diario.add(pagina);
    }

    /**
     * incrementa il saldo di un valore specificato.
     */
    public void addSaldo(double valore) {
        saldo += valore;
    }

    /**
     * paga una multa di un valore specificato.
     */
    public void pagaMulta(double valore) {
        paga(valore);
    }

    /**
     * aumenta il numero di mazzette accettate dalla polizia.
     */
    public void incrementaMazzetteAccettatePolizia() {
        numeroMazzetteAccettatePolizia++;
        if (numeroMazzetteAccettatePolizia >= 2) {
            statoGioco = StatoGioco.GIUSTIZIATO;
        }
    }

    /**
     * aumenta il numero di rivoluzionari accettati.
     */
    public void incrementaRivoluzionariAccettati() {
        numeroRivoluzionariAccettati++;
        if (numeroRivoluzionariAccettati >= 5) {
            statoGioco = StatoGioco.RIVOLUZIONE;
        }
    }

    /**
     * getter per il nome del personaggio.
     *
     * @return il nome del personaggio
     */
    public String getNomePersonaggio() {
        return nomePersonaggio;
    }

    /**
     * getter per lo stato del gioco.
     *
     * @return lo stato del gioco
     */
    public StatoGioco getStatoGioco() {
        return statoGioco;
    }

    /**
     * getter per il giorno corrente del gioco.
     *
     * @return il giorno corrente del gioco
     */
    public int getGiorno() {
        return giorno;
    }

    /**
     * aumenta il giorno corrente e aggiorna il saldo.
     *
     * @param personeControllate la lista delle persone controllate
     */
    public void incrementaGiorno(ArrayList<PersonaControllata> personeControllate) {
        if (!personeControllate.isEmpty()) {
            addSaldo(50 * personeControllate.size());
        }
        paga(200);

        PaginaDiario pagina = new PaginaDiario(String.format("%d-%02d-%02d", data.getYear(), data.getMonthValue(), data.getDayOfMonth()), saldo);
        for (PersonaControllata persona : personeControllate) {
            pagina.addPersonaControllata(persona);
        }
        diario.add(pagina);

        giorno++;

        data = LocalDate.of(data.getYear(), data.getMonthValue(), data.getDayOfMonth() + 1);

        if (giorno > 12 && statoGioco == StatoGioco.IN_CORSO && saldo > 2300) {
            statoGioco = StatoGioco.DODICESIMO_GIORNO;
        } else if (giorno > 12 && statoGioco == StatoGioco.IN_CORSO) {
            statoGioco = StatoGioco.NON_ABBASTANZA_SOLDI;
        }
    }

    /**
     * fa pagare un valore specificato al saldo.
     */
    private void paga(double valore) {
        saldo -= valore;
        if (saldo < 0) {
            statoGioco = StatoGioco.ARRESTATO;
        }
    }

    /**
     * getter per la data del gioco.
     *
     * @return la data del gioco
     */
    public LocalDate getData() {
        return data;
    }

    /**
     * salva il diario in un file.
     *
     */
    public void saveDiario() {
        Parser.saveDiario("FileEsercizio/diario", diario, formato);
    }
}
