package it.unibs.fp.patenteelibbrettopregoh;

import com.google.gson.annotations.SerializedName;

/**
 * La classe PersonaControllata estende la classe Persona e rappresenta una persona che è stata controllata.
 * Contiene informazioni sul risultato del controllo e sul tipo di persona controllata.
 */
public class PersonaControllata extends Persona {
    @SerializedName("approvato")
    private final boolean esitoControllo;
    private transient final TipoPersonaControllata tipoPersonaControllata;

    /**
     * Costruttore della classe PersonaControllata.
     *
     * @param persona L'oggetto Persona da cui estrarre le informazioni.
     * @param esitoControllo Il risultato del controllo (true se approvato, false se rifiutato).
     * @param tipoPersonaControllata Il tipo di persona controllata.
     */
    public PersonaControllata(Persona persona, boolean esitoControllo, TipoPersonaControllata tipoPersonaControllata) {
        super(persona.getNome(), persona.getCognome(), persona.getSesso(), persona.getDataNascita(), persona.getComuneNascita(), persona.getCodiceFiscale(), persona.getScadenzaDocumento());
        this.esitoControllo = esitoControllo;
        this.tipoPersonaControllata = tipoPersonaControllata;
    }

    /**
     * Restituisce il risultato del controllo.
     *
     * @return true se il controllo è positivo, false altrimenti.
     */
    public boolean isEsitoControllo() {
        return esitoControllo;
    }

    /**
     * Restituisce il tipo di persona controllata.
     *
     * @return Il tipo di persona controllata.
     */
    public TipoPersonaControllata getTipoPersonaControllata() {
        return tipoPersonaControllata;
    }

    /**
     * Restituisce lo stato del transito in base all'esito del controllo.
     *
     * @return "APPROVATO" se il controllo è positivo, "RIFIUTATO" altrimenti.
     */
    public String getStatoTransito() {
        return esitoControllo ? "APPROVATO" : "RIFIUTATO";
    }

    /**
     * Restituisce una rappresentazione testuale della PersonaControllata.
     *
     * @return Una stringa contenente le informazioni della persona controllata.
     */
    @Override
    public String toString() {
        return "Nome: " + getNome() + "\n" +
                "Cognome: " + getCognome() + "\n" +
                "Sesso: " + getSesso() + "\n" +
                "Data di Nascita: " + getDataNascita() + "\n" +
                "Comune di Nascita: " + getComuneNascita() + "\n" +
                "Codice Fiscale: " + getCodiceFiscale() + "\n" +
                "Scadenza Documento: " + getScadenzaDocumento() + "\n" +
                "Esito Controllo: " + (esitoControllo ? "Positivo" : "Negativo") + "\n" +
                "Tipo Persona Controllata: " + tipoPersonaControllata.toString() + "\n";
    }
}
